@@ -24,8 +24,10 @@ const Utils = imports.utils;
 
 const MAX_RETRIES = 3;
 
-const TP_ERROR_PREFIX = 'org.freedesktop.Telepathy.Error.'
-const TP_ERROR_ALREADY_CONNECTED = TP_ERROR_PREFIX + 'AlreadyConnected';
+const ConnectionError = {
+    CANCELLED: Tp.error_get_dbus_name(Tp.Error.CANCELLED),
+    ALREADY_CONNECTED: Tp.error_get_dbus_name(Tp.Error.ALREADY_CONNECTED)
+};
 
 const Application = new Lang.Class({
     Name: 'Application',
@@ -36,7 +38,7 @@ const Application = new Lang.Class({
 
         Gettext.bindtextdomain('polari', Config.LOCALE_DIR);
         Gettext.textdomain('polari');
-        GLib.set_prgname('polari');
+        GLib.set_prgname('org.gnome.Polari');
         GLib.set_application_name('Polari');
         this._window = null;
         this._pendingRequests = {};
@@ -53,8 +55,6 @@ const Application = new Lang.Class({
         window.C_ = Gettext.pgettext;
         window.ngettext = Gettext.ngettext;
 
-        Gtk.init(null);
-
         let w = new Polari.FixedSizeFrame(); // register gtype
         w.destroy();
 
@@ -66,7 +66,7 @@ const Application = new Lang.Class({
                 this._removeSavedChannelsForAccount(account);
             }));
 
-        this._settings = new Gio.Settings({ schema: 'org.gnome.polari' });
+        this._settings = new Gio.Settings({ schema_id: 'org.gnome.polari' });
 
         this.pasteManager = new PasteManager.PasteManager();
         this.notificationQueue = new AppNotifications.NotificationQueue();
@@ -78,11 +78,11 @@ const Application = new Lang.Class({
             state: GLib.Variant.new('b', false) },
           { name: 'show-join-dialog',
             activate: Lang.bind(this, this._onShowJoinDialog),
-            accel: '<Primary>n' },
+            accels: ['<Primary>n'] },
           { name: 'show-message-user-dialog',
             activate: Lang.bind(this, this._onShowMessageUserDialog),
             create_hook: Lang.bind(this, this._accountActionsCreateHook),
-            accel: '<Primary>m' },
+            accels: ['<Primary>m'] },
           { name: 'join-room',
             activate: Lang.bind(this, this._onJoinRoom),
             parameter_type: GLib.VariantType.new('(ssu)') },
@@ -95,13 +95,13 @@ const Application = new Lang.Class({
           { name: 'leave-current-room',
             activate: Lang.bind(this, this._onLeaveCurrentRoom),
             create_hook: Lang.bind(this, this._leaveRoomCreateHook),
-            accel: '<Primary>w' },
+            accels: ['<Primary>w'] },
           { name: 'leave-selected-rooms' },
           { name: 'user-list',
             activate: Lang.bind(this, this._onToggleAction),
             create_hook: Lang.bind(this, this._userListCreateHook),
             state: GLib.Variant.new('b', false),
-            accel: 'F9' },
+            accels: ['F9', '<Primary>u'] },
           { name: 'selection-mode',
             activate: Lang.bind(this, this._onToggleAction),
             create_hook: Lang.bind(this, this._selectionModeHook),
@@ -114,15 +114,15 @@ const Application = new Lang.Class({
             activate: Lang.bind(this, this._onShowAbout) },
           { name: 'quit',
             activate: Lang.bind(this, this._onQuit),
-            accel: '<Primary>q' },
+            accels: ['<Primary>q'] },
           { name: 'next-room',
-            accel: '<Primary>Page_Down' },
+            accels: ['<Primary>Page_Down', '<Alt>Down'] },
           { name: 'previous-room',
-            accel: '<Primary>Page_Up' },
+            accels: ['<Primary>Page_Up', '<Alt>Up'] },
           { name: 'first-room',
-            accel: '<Primary>Home' },
+            accels: ['<Primary>Home'] },
           { name: 'last-room',
-            accel: '<Primary>End' },
+            accels: ['<Primary>End'] },
           { name: 'nth-room',
             parameter_type: GLib.VariantType.new('i') }
         ];
@@ -141,41 +141,28 @@ const Application = new Lang.Class({
                     action.connect('activate', actionEntry.activate);
                 if (actionEntry.change_state)
                     action.connect('change-state', actionEntry.change_state);
-                if (actionEntry.accel)
-                    this.add_accelerator(actionEntry.accel,
-                                         'app.' + actionEntry.name, null);
+                if (actionEntry.accels)
+                    this.set_accels_for_action('app.' + actionEntry.name,
+                                               actionEntry.accels);
                 this.add_action(action);
         }));
 
         for (let i = 1; i < 10; i++)
-            this.add_accelerator('<Alt>' + i, 'app.nth-room',
-                                 GLib.Variant.new('i', i));
-
-        this._window = new MainWindow.MainWindow(this);
-        this._window.window.connect('destroy', Lang.bind(this,
-            function() {
-                for (let id in this._pendingRequests)
-                    this._pendingRequests[id].cancellable.cancel();
-                this.emitJS('prepare-shutdown');
-            }));
-
-        let provider = new Gtk.CssProvider();
-        let uri = 'resource:///org/gnome/Polari/application.css';
-        let file = Gio.File.new_for_uri(uri);
-        try {
-            provider.load_from_file(Gio.File.new_for_uri(uri));
-        } catch(e) {
-            logError(e, "Failed to add application style");
-        }
-        Gtk.StyleContext.add_provider_for_screen(this._window.window.get_screen(),
-                                                 provider, 600);
-
-        this._window.window.show_all();
+            this.set_accels_for_action('app.nth-room(%d)'.format(i), ['<Alt>' + i]);
     },
 
     vfunc_activate: function() {
-        if (this._window)
-            this._window.window.present();
+        if (!this._window) {
+            this._window = new MainWindow.MainWindow(this);
+            this._window.window.connect('destroy', Lang.bind(this,
+                function() {
+                    for (let id in this._pendingRequests)
+                        this._pendingRequests[id].cancellable.cancel();
+                    this.emitJS('prepare-shutdown');
+            }));
+            this._window.window.show_all();
+        }
+        this._window.window.present();
     },
 
     _updateAccountAction: function(action) {
@@ -335,6 +322,19 @@ const Application = new Lang.Class({
                                            this._onEnsureChannel, requestData));
     },
 
+    _retryRequest: function(requestData) {
+        let account = requestData.account;
+
+        // Try again with a different nick
+        let params = account.dup_parameters_vardict().deep_unpack();
+        let oldNick = params['account'].deep_unpack();
+        let nick = oldNick + '_';
+        this._updateAccountName(account, nick, Lang.bind(this,
+            function() {
+                this._ensureChannel(requestData);
+            }));
+    },
+
     _onEnsureChannel: function(req, res, requestData) {
         let account = req.account;
 
@@ -344,24 +344,15 @@ const Application = new Lang.Class({
             if (requestData.targetHandleType == Tp.HandleType.ROOM)
                 this._addSavedChannel(account, requestData.targetId);
         } catch (e if e.matches(Tp.Error, Tp.Error.DISCONNECTED)) {
-            let [error,] = account.dup_detailed_error_vardict();
-            if (error != TP_ERROR_ALREADY_CONNECTED)
-                throw(e);
-
-            if (++requestData.retry >= MAX_RETRIES) {
-                throw(e);
-                return;
+            let error = account.connection_error;
+            if (error == ConnectionError.ALREADY_CONNECTED &&
+                requestData.retry++ < MAX_RETRIES) {
+                    this._retryRequest(requestData);
+                    return;
             }
 
-            // Try again with a different nick
-            let params = account.dup_parameters_vardict().deep_unpack();
-            let oldNick = params['account'].deep_unpack();
-            let nick = oldNick + '_';
-            this._updateAccountName(account, nick, Lang.bind(this,
-                function() {
-                    this._ensureChannel(requestData);
-                }));
-            return;
+            if (error && error != ConnectionError.CANCELLED)
+                logError(e);
         } catch (e if e.matches(Tp.Error, Tp.Error.CANCELLED)) {
             // interrupted by user request, don't log
         } catch (e) {
